@@ -20,6 +20,10 @@ class GcodeLine:
         else:
             self.type = Commands.OTHER
 
+    @property
+    def command_str(self):
+        return f"{self.command[0]}{self.command[1] if self.command[1] is not None else ''}"
+
     def get_param(self, param: str, return_type=None, default=None):
         """
         Returns the value of the param if it exists, otherwise it will the default value.
@@ -46,10 +50,13 @@ class GcodeLine:
             return
         self.params.pop(param)
 
-    def to_gcode(self):
-        command = f"{self.command[0]}{self.command[1]}"
+    @property
+    def gcode_str(self):
+        command = self.command_str
         params = " ".join(f"{param}{self.get_param(param)}" for param in self.params.keys())
         comment = f"; {self.comment}" if self.comment != '' else ""
+        if command == ';':
+            return comment
         return f"{command} {params} {comment}".strip()
 
 
@@ -59,24 +66,20 @@ class GcodeParser:
         self.lines: List[GcodeLine] = get_lines(self.gcode, include_comments)
         self.include_comments = include_comments
 
-    def __repr__(self):
-        including_comments = " including comments" if self.include_comments else ""
-        return f"<GcodeParser: {len(self.lines)} lines{including_comments}>"
-
 
 def get_lines(gcode, include_comments=False):
-    regex = r'(?!; *.+)(G|M|T|g|m|t)(\d+)(( *(?!G|M|g|m)\w-?\d+\.?\d*)*) *\t*(; *(.*))?|; *(.+)'
+    regex = r'(?!; *.+)(G|M|T|g|m|t)(\d+)(([ \t]*(?!G|M|g|m)\w(".*"|([-\d\.]*)))*)[ \t]*(;[ \t]*(.*))?|;[ \t]*(.+)'
     regex_lines = re.findall(regex, gcode)
     lines = []
     for line in regex_lines:
         if line[0]:
             command = (line[0].upper(), int(line[1]))
-            comment = line[5]
+            comment = line[-2]
             params = split_params(line[2])
 
         elif include_comments:
             command = (';', None)
-            comment = line[6]
+            comment = line[-1]
             params = {}
 
         else:
@@ -86,25 +89,28 @@ def get_lines(gcode, include_comments=False):
             GcodeLine(
                 command=command,
                 params=params,
-                comment=comment,
+                comment=comment.strip(),
             ))
 
     return lines
 
 
-def is_float(num: str):
-    if re.search(r'[+-]?\d+.\d+', num):
-        return True
-    return False
+def element_type(element: str):
+    if re.search(r'"', element):
+        return str
+    if re.search(r'\..*\.', element):
+        return str
+    if re.search(r'[+-]?\d+\.\d+', element):
+        return float
+    return int
 
 
 def split_params(line):
-    regex = r'((?!\d)\w+?)(-?\d+\.?\d*)'
+    regex = r'((?!\d)\w+?)(".*"|(\d+\.?)+|-?\d+\.?\d*)'
     elements = re.findall(regex, line)
     params = {}
     for element in elements:
-        element_type = float if is_float(element[1]) else int
-        params[element[0].upper()] = element_type(element[1])
+        params[element[0].upper()] = element_type(element[1])(element[1])
 
     return params
 
